using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Collections.Concurrent;

namespace Utility
{
    public class RandomHelp
    {
        static volatile Int32 count = 0;
        static Random GetRandom()
        {
            count++;
            count += (int)DateTime.Now.TotalSecond();
            return new Random(count);
        }

        /// <summary>
        /// 根据权重随机对象
        /// </summary>
        /// <typeparam name="TKey">对象</typeparam>
        /// <param name="weightDic">k=对象，v=权重</param>
        /// <returns></returns>
        public static TKey GetRandomTypeByWeight<TKey>(Dictionary<TKey, int> weightDic)
        {
            int totalWeight = 0;
            foreach (var weight in weightDic.Values)
            {
                totalWeight += weight;
            }

            var ran = GetRandom();
            int randomResult = ran.Next(0, totalWeight);

            foreach (var item in weightDic)
            {
                randomResult -= item.Value;
                if (randomResult < 0)
                {
                    return item.Key;
                }
            }

            return default(TKey);
        }

        /// <summary>
        /// 是否命中
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static bool IsHit(int value)
        {
            var ran = GetRandom();
            return ran.RandomInt(100) < value;
        }

        public static bool IsHit(double value)
        {
            var ran = GetRandom();
            return ran.RandomDouble(1) < value;
        }

        /// <summary>
        /// 随机一个小于maxValue的Int值
        /// </summary>
        /// <param name="self"></param>
        /// <param name="dMax"></param>
        /// <returns></returns>
        public static int RandomInt(int maxValue)
        {
            var ran = GetRandom();
            return ran.RandomInt(maxValue);
        }

        /// <summary>
        /// 随机一个大于等于minValue小于等于maxValue的Int值
        /// </summary>
        /// <param name="self"></param>
        /// <param name="dMax"></param>
        /// <returns></returns>
        public static int RandomInt(int minValue, int maxValue)
        {
            var ran = GetRandom();
            return ran.RandomInt(minValue, maxValue + 1);
        }

        public static double RandomDouble(double maxValue)
        {
            var ran = GetRandom();
            return ran.RandomDouble(maxValue);
        }

        public static double RandomDouble(double minValue, double maxValue)
        {
            var ran = GetRandom();
            return ran.RandomDouble(minValue, maxValue);
        }

        public static float RandomFloat(float maxValue)
        {
            var ran = GetRandom();
            return ran.RandomFloat(maxValue);
        }

        public static float RandomFloat(float minValue, float maxValue)
        {
            var ran = GetRandom();
            return ran.RandomFloat(minValue, maxValue);
        }

        public static int RandomSeed(ref int seed)
        {
            seed = (seed * 9301 + 49297) % 233280;
            return (int)(seed / (233280.0) * 100) | 0;
        }

    }
}
